﻿using RenovationWorkContracts.BindingModels;
using RenovationWorkContracts.ViewModels;
using RenovationWorkDataModels.Enums;
using RenovationWorkDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace RenovationWorkFileImplement.Models
{
    internal class Order : IOrderModel
    {
        public int PackageId { get; private set; }

        public string PackageName { get; private set; } = string.Empty;

        public int Count { get; private set; }

        public double Sum { get; private set; }

        public OrderStatus Status { get; private set; } = OrderStatus.Неизвестен;

        public DateTime DateCreate { get; private set; } = DateTime.Now;

        public DateTime? DateImplement { get; private set; }

        public int Id { get; private set; }

        public static Order? Create(OrderBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                PackageId = model.PackageId,
                PackageName = model.PackageName,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement
            };
        }

        public static Order? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }

            var order = new Order()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                PackageId = Convert.ToInt32(element.Element("PackageId")!.Value),
                PackageName = element.Element("PackageName")!.Value,
                Count = Convert.ToInt32(element.Element("Count")!.Value),
                Sum = Convert.ToDouble(element.Element("Sum")!.Value),
                Status = (OrderStatus)Enum.Parse(typeof(OrderStatus), element.Element("Status")!.Value),
                DateCreate = DateTime.ParseExact(element.Element("DateCreate")!.Value, "G", null)
            };

            DateTime.TryParse(element.Element("DateImplement")!.Value, out DateTime dateImpl);
            order.DateImplement = dateImpl;

            return order;
        }

        public void Update(OrderBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            PackageId = model.PackageId;
            PackageName = model.PackageName;
            Count = model.Count;
            Sum = model.Sum;
            Status = model.Status;
            DateCreate = model.DateCreate;
            DateImplement = model.DateImplement;
        }

        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            PackageId = PackageId,
            PackageName = PackageName,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement
        };

        public XElement GetXElement => new("Order",
            new XAttribute("Id", Id),
            new XElement("PackageName", PackageName),
            new XElement("PackageId", PackageId.ToString()),
            new XElement("Count", Count.ToString()),
            new XElement("Sum", Sum.ToString()),
            new XElement("Status", Status.ToString()),
            new XElement("DateCreate", DateCreate.ToString()),
            new XElement("DateImplement", DateImplement.ToString()));
    }
}
